'''
# triangulate_ui.py
#
# Created by Danny Wynne
#
# in maya run:

import triangulate_ui as tui
reload(tui)
tui.main()

#
#
'''
from PyQt4 import QtGui, QtCore, uic
import sip
import os
import maya.cmds as cmds
import maya.mel as mel
import maya.OpenMaya as om
import maya.OpenMayaUI as mui
import dw_triangulate as tri
reload(tri)

def getMayaWindow():
	ptr = mui.MQtUtil.mainWindow()
	return sip.wrapinstance(long(ptr), QtCore.QObject)

user_maya_dir = os.environ["MAYA_APP_DIR"]
uiFile = os.path.normpath(os.path.join(user_maya_dir, r"scripts\dw_triangulate_ui.ui"))

form_class, base_class = uic.loadUiType(uiFile)
	
class TriTest(base_class, form_class):
	def __init__ (self, parent=getMayaWindow()):
		super(base_class, self).__init__(parent)
		self.setupUi(self)
		self.setObjectName("triangulate_test")
		self.setWindowTitle("triangulage test")
		self.curves=[]
		self.ref_curves=[]
		self.slider_x.sliderMoved.connect(self.slider_moved)
		self.slider_y.sliderMoved.connect(self.slider_moved)
		self.slider_z.sliderMoved.connect(self.slider_moved)
		self.update_btn.clicked.connect(self.update)
		self.ref_a_loc = cmds.spaceLocator(name="ref_a_loc")[0]
		self.ref_b_loc = cmds.spaceLocator(name="ref_b_loc")[0]
		self.ref_c_loc = cmds.spaceLocator(name="ref_c_loc")[0]
		self.ref_origin_loc = cmds.spaceLocator(name="ref_origin_loc")[0]
		self.ref_target_loc = cmds.polySphere(r=.5, sx=10, sy=10, ch=0)[0]
		
		cmds.setAttr(self.ref_a_loc+".t", 12,1,0)
		cmds.setAttr(self.ref_b_loc+".t", 2,16,2)
		cmds.setAttr(self.ref_c_loc+".t", -3,.5,7)
		
		self.a_loc = cmds.spaceLocator(name="a_loc")[0]
		self.b_loc = cmds.spaceLocator(name="b_loc")[0]
		self.c_loc = cmds.spaceLocator(name="c_loc")[0]
		self.origin_loc = cmds.spaceLocator(name = "origin_loc")[0]
		self.target_loc = cmds.polySphere(r=.5, sx=10, sy=10, ch=0)[0]
		
		cmds.setAttr(self.a_loc+".t", 12.1,0,0)
		cmds.setAttr(self.b_loc+".t", 2,16.1, 2)
		cmds.setAttr(self.c_loc+".t", -7, 1, 7.1)
		locs = [self.a_loc, self.b_loc, self.c_loc, self.origin_loc, self.target_loc]
		for l in locs:
			cmds.setAttr(l+".overrideEnabled", 1)
			cmds.setAttr(l+".overrideColor", 12)
	def slider_moved(self):
		x = self.slider_x.value()
		y = self.slider_y.value()
		z = self.slider_z.value()
		x = float(x)/10.0
		y = float(y)/10.0
		z = float(z)/10.0
		cmds.move(x,y,z, self.ref_target_loc, a = 1)
		self.update()
	
	def update(self):
		if cmds.objExists("debug_grp"): cmds.delete("debug_grp")
		cmds.group(em=1, name="debug_grp")
		base_refs = TriData(pnt(self.ref_a_loc), pnt(self.ref_b_loc), pnt(self.ref_c_loc))
		base_ref_origin = pnt(self.ref_origin_loc)
		base_ref_target = pnt(self.ref_target_loc)
		
		base_vecs = TriData((base_refs.a-base_ref_origin), (base_refs.b-base_ref_origin), (base_refs.c-base_ref_origin))
		params = tri.get_ref_params(base_ref_origin, base_ref_target, base_refs)
		
		ref_pnts = TriData(pnt(self.a_loc), pnt(self.b_loc), pnt(self.c_loc))
		origin = pnt(self.origin_loc)
		
		self.draw_axis(base_ref_origin, base_ref_target, base_refs, params, 0)
		triangulate = tri.Triangulate(origin, ref_pnts, params, base_vecs, debug = self.debug_cb.isChecked())
		if type(triangulate.pnt).__name__ == "MPoint":
			cmds.setAttr(self.target_loc+".t", triangulate.pnt.x, triangulate.pnt.y, triangulate.pnt.z)
			self.draw_axis(origin, triangulate.pnt, ref_pnts, params, 12)
		cmds.select(cl=1)
		return 
		
	def draw_axis(self, origin, target, pnts, params, color=3):
		line_list = [[pnts.a, params.a], [pnts.b, params.b], [pnts.c, params.c]]
		for line in line_list:
			pnt_a = origin+((line[0]-origin)*line[1])
			crv = cmds.curve(p=((pnt_a.x, pnt_a.y, pnt_a.z),(target.x, target.y,target.z)), d=1)
			cmds.setAttr(crv+".overrideEnabled", 1)
			cmds.setAttr(crv+".overrideColor", 3)
			cmds.parent(crv, "debug_grp")
			crv = cmds.curve(p=((origin.x, origin.y, origin.z),(line[0].x, line[0].y,line[0].z)), d=1)
			cmds.setAttr(crv+".overrideEnabled", 1)
			cmds.setAttr(crv+".overrideColor", 3)
			cmds.parent(crv, "debug_grp")
			
class TriData(object):
	def __init__(self, a,b,c):
		self.a, self.b, self.c = a,b,c
		
def pnt(obj):
	return om.MPoint(*cmds.xform(obj, q=1, piv=1, ws=1)[0:3])

def main():
	global tri_test
	try: tri_test.close()
	except:pass
	tri_test = TriTest()
	tri_test.show()
